/*******************************************************************
* ARexx Script: Alarmist_Warn.rexx                                 *
* Version.....: 2.00 (23.08.96)                                    *
* ---------------------------------------------------------------- *
* This script can be used in the Alarmist Pre-Warn feature.        *
* Just select this script in the Warn Configuration and select     *
* the execution type as 'ARexx.'                                   *
*                                                                  *
* If Alarmist calls this script on a Warn event it will pass the   *
* actual number of the Alarm that it is acting on as an argument   *
* (see first line below). The script uses this number to get       *
* information from the list about this Alarm event. If, for some   *
* reason there is no argument passed to the script, it will ask    *
* Alarmist for the next Warn event number. This latter method is   *
* not accurate in all situations because if multiple Warn events   *
* occur at the same time then by the time the script asks Alarmist *
* for the number of the next Warn event, the number might have     *
* changed. So Alarmist gives the actual Alarm number when calling  *
* the script as an actual Warn event.                              *
*                                                                  *
* The resultant information is outputed using the program "talk".  *
*******************************************************************/

options results

event = arg(1)

/* if we were not given an event number then then quit */
if (event = '' | event = 0) then
do
	address ALARMIST

	getnextwarn
	event = result

	if (event = '' | event = 0) then
	do
		call Talk('no more warnings for today')

		exit(0)
	end
end

/* get computer time */
clock_minutes = time(m);

/* get name */
getfield entry event 1
name = result

/* get hour */
getfield entry event 3
hour = result

/* get minute */
getfield entry event 4
minute = result

alarm_minutes = hour * 60 + minute

/* calculate difference between alarm and computer time */
diff_minutes = alarm_minutes - clock_minutes

diff_hours   = diff_minutes % 60
diff_minutes = diff_minutes // 60
output = ''

/* put together the output string */
if (diff_hours > 0) then
do
	output = '%n'diff_hours' hour'
	
	if (diff_hours > 1) then
		output = output || 's'
end

if (diff_minutes > 0) then
do
	output = output '%n'diff_minutes' min ut'
	
	if (diff_minutes > 1) then
		output = output || 's'
end

/* Talk to the user */
call Talk('->2 'name' will a cur in 'output)

/* It's as easy as that ! */

exit (0)

/*********************************************/
/* Check for TALK ARexx port or call program */
/*********************************************/

Talk: PROCEDURE

parse ARG speech

if (show('p', 'TALK')) then
do
	address TALK
	
	speech
end
else
do
	address command
	
	c:talk speech
end

return (0)

/*********************************************/

/* End */