/*******************************************************************
* ARexx Script: Locate_CD_Stats.rexx                               *
* Version.....: 1.01 (24.02.97)                                    *
* ---------------------------------------------------------------- *
* This is script is to be used with the Locate data file: 'CDs.loc'*
* It scans the currently loaded list of CDs and spits out some     *
* very trivial information indeed 8). You can install it into an   *
* ARexx menu location or just run it as a new script from Locate.  *                            
*******************************************************************/

options results

dollar = '24'X

getentries
entries = result

i = 1
total_tracks = 0
total_mins   = 0
total_secs   = 0
longest      = 0
shortest     = 0
longest_average_cd_secs = 0
shortest_average_cd_secs = 0

do while (i <= entries)

	'getfield entry 'i' 6'
	tracks = result
	total_tracks = total_tracks + tracks
		
	'getfield entry 'i' 10'
	data = result

	if (data ~= '') then
		parse var data min '.' sec
	else
	do
		min = 0
		sec = 0
	end

	total_mins = total_mins + min
	total_secs = total_secs + sec

	length = min * 60 + sec
	
	if (length > longest) then
	do
		longest = length
		longest_cd_secs = sec
		longest_cd_mins = min
		'getfield entry 'i' 1'
		longest_cd_band = result
		'getfield entry 'i' 2'
		longest_cd_album = result
	end
	
	if (length < shortest) then
	do
		shortest = length
		shortest_cd_secs = sec
		shortest_cd_mins = min
		'getfield entry 'i' 1'
		shortest_cd_band = result
		'getfield entry 'i' 2'
		shortest_cd_album = result
	end
	
	if (shortest = 0) then
		shortest = length
	
	average = length / tracks
	
	if (average > longest_average_cd_secs) then
	do
		longest_average_cd_secs = average
		'getfield entry 'i' 1'
		longest_average_cd_band = result
		'getfield entry 'i' 2'
		longest_average_cd_album = result
	end
	
	if (average < shortest_average_cd_secs) then
	do
		shortest_average_cd_secs = average
		'getfield entry 'i' 1'
		shortest_average_cd_band = result
		'getfield entry 'i' 2'
		shortest_average_cd_album = result
	end
	
	if (shortest_average_cd_secs = 0) then
		shortest_average_cd_secs = average
	
	i = i + 1
end

/* Total Length of all CDs */
absolute_secs  = total_mins * 60 + total_secs

   s = absolute_secs
relative_days  = s % 86400
   s = s // 86400
relative_hours = s % 3600
   s = s // 3600
relative_mins  = s % 60
   s = s // 60
relative_secs  = s

/* Average CD Length */
absolute_average_cd_secs = absolute_secs / entries
relative_average_cd_mins = trunc((absolute_average_cd_secs / 60), 0)
relative_average_cd_secs = round((absolute_average_cd_secs // 60), 3)

/* Average Song Length */
absolute_average_song_secs = absolute_secs / total_tracks
relative_average_song_mins = trunc((absolute_average_song_secs / 60), 0)
relative_average_song_secs = round((absolute_average_song_secs // 60), 3)

/* Longest Average Song Length */
longest_average_cd_mins = trunc((longest_average_cd_secs / 60), 0)
longest_average_cd_secs = round((longest_average_cd_secs // 60), 3)

/* Shortest Average Song Length */
shortest_average_cd_mins = trunc((shortest_average_cd_secs / 60), 0)
shortest_average_cd_secs = round((shortest_average_cd_secs // 60), 3)

/* Estimated costs */
total_cost      = entries * 25
cost_per_song   = round((total_cost / total_tracks), 2)
cost_per_second = round((total_cost / absolute_secs), 4)
cost_per_minute = round((total_cost / (absolute_secs / 60)), 3)
cost_per_hour   = round((total_cost / (absolute_secs / 3600)), 3)
cost_per_day    = round((total_cost / (absolute_secs / 86400)), 3)

say 'Total CDs...: 'entries
say 'Total Tracks: 'total_tracks
say ''
say 'Estimated cost (at 'dollar'25)..: 'dollar || total_cost
say 'Estimated cost per song..: 'dollar || cost_per_song
say ''
say 'Estimated cost per second: 'dollar || cost_per_second
say 'Estimated cost per minute: 'dollar || cost_per_minute
say 'Estimated cost per hour..: 'dollar || cost_per_hour
say 'Estimated cost per day...: 'dollar || cost_per_day
say ''
say 'Total Length of all CDs: 'relative_days' days, 'relative_hours' hours, 'relative_mins' mins, 'relative_secs' sec'
say 'Average CD Length......: 'relative_average_cd_mins' min, 'relative_average_cd_secs' sec'
say 'Average Song Length....: 'relative_average_song_mins' min, 'relative_average_song_secs' sec'
say ''
say 'Shortest CD:'
say '------------------------------'
say '   Band..: 'shortest_cd_band
say '   Album.: 'shortest_cd_album
say '   Length: 'shortest_cd_mins' min, 'shortest_cd_secs' sec'
say ''
say 'Longest CD:'
say '------------------------------'
say '   Band..: 'longest_cd_band
say '   Album.: 'longest_cd_album
say '   Length: 'longest_cd_mins' min, 'longest_cd_secs' sec'
say ''
say 'CD With Shortest Song Average:'
say '------------------------------'
say '   Band..: 'shortest_average_cd_band
say '   Album.: 'shortest_average_cd_album
say '   Length: 'shortest_average_cd_mins' min, 'shortest_average_cd_secs' sec'
say ''
say 'CD With Longest Song Average:'
say '------------------------------'
say '   Band..: 'longest_average_cd_band
say '   Album.: 'longest_average_cd_album
say '   Length: 'longest_average_cd_mins' min, 'longest_average_cd_secs' sec'
say ''

exit (0)

/*******************************************/

Round: PROCEDURE

ARG value,decimals .

if (decimals > 0) then
	decimals = 10 ** (decimals)
else
	decimals = 1

return (trunc(value * decimals + 0.5) / decimals)

/*******************************************/

/* End */
