/*******************************************************************
* ARexx Script: OnTime_QuickEvent.rexx                             *
* Version.....: 1.00 (2.03.97)                                     *
* ---------------------------------------------------------------- *
* This script provides and example of how to create a custom Event *
* in On-Time very quickly using ARexx. You can tailor this script  *
* to suit a type of Event that you may enter regularly. You could  *
* call this script from the On-Time ARexx menu or you could assign *
* it to a Hot-Key using a program like Tool Manager.               *
*******************************************************************/

options results

/* Make sure we're talking to On-Time */
address ONTIME

/* Today's Julian */
'dateinfo julian today'
julian_today = result

/* Setup some items for the Event.
   We'll set the begin date of the event for two weeks from now. This could
   be done by specifying the string "2 weeks" as a date, but the word "week"
   might be non-english, if there is a non-english catalog in use.

   We've set all the possible parameters for an Event but there is NO NEED in
   this example. The extra parameters are just there as examples.
*/

name          = "Do: Return Library Book"
notes         = "Late returns cost $1 each day."
begin_date    = julian_today + 14
end_date      = '' /* No end date */
begin_time    = '' /* This means 'no time' coz '0' = '12:00am' */
end_time      = ''
repeat_type   = 0
repeat_amount = 0
repeat_same   = 0  /* Repeat on same day and week of month */
repeat_until  = '' /* No repeat until */
deletewhenold = 0
passive       = 0
remind_days   = 7
alarm_exec_type    = 0
alarm_exec_string  = ''
snooze_exec_type   = 0
snooze_exec_string = ''
snooze_seconds     = 0
snooze_repeats     = 0
everyday_alarm  = 0
end_alarm       = 0
alarm_log_popup = 0
alarm_warn_type = 0
alarm_warn_time = 0

/* Create the new event */
new event

/* Turn the Event list redraw/rebuild OFF.
   We should do this after the Event is added because we need to find
   the active event in the Filter List */
listquiet event state 1

/* Setup the event with our new settings.
   We won't specify and event number because the one we just inserted should
   still be active */
setfield  1 name
setfield  2 notes
setfield  3 begin_date
setfield  4 end_date
setfield  5 begin_time
setfield  6 end_time
setfield  7 repeat_type
setfield  8 repeat_amount
setfield  9 repeat_same
setfield 10 repeat_until
setfield 11 deletewhenold
setfield 12 passive
setfield 13 remind_days
setfield 14 alarm_exec_type
setfield 15 alarm_exec_string
setfield 16 snooze_exec_type
setfield 17 snooze_exec_string
setfield 18 snooze_seconds
setfield 19 snooze_repeats
setfield 20 everyday_alarm
setfield 21 end_alarm
setfield 22 alarm_log_popup
setfield 23 alarm_warn_type
setfield 24 alarm_warn_time

/* Release the list display lock */
listquiet event state 0

/* The event is now setup but we might want to manually set some things */
/* Event Edit Window */
edit event

/* Event Alarm Window (if this Event has a time of day) */
if (begin_time > 0 | end_time > 0) then
	edit alarm

/* End */
