/*******************************************************************
* ARexx Script: Talk_ARexxPort.rexx                                *
* Version.....: 1.01 (05.03.97)                                    *
* ---------------------------------------------------------------- *
* This script exists as a solution to a problem that occurs when   *
* using the program Talk. If you call Talk two or more times (at   *
* the same time) it can get confused, especially if they open Face *
* Windows. This script will open an ARexx port and any messages it *
* receives will be queued and spoken by the program Talk.          *
* You may provide a single argument when calling the script. This  *
* will be the name of the port to be opened. Otherwise it will use *
* the default name 'TALK' (the recommended name!).                 *
*                                                                  *
* For example: Shell> rx Everyday:rexx/Talk_ARexxPort.rexx         *
*              Shell> rx Everyday:rexx/Talk_ARexxPort.rexx pname   *
*                                                                  *
* To quit this script and close the port, just send it a 'quit'.   *
* For example: Shell> rx "address Talk quit                        *
*                                                                  *
* The Everyday Organiser scripts are setup to take advantage of    *
* this script. When they're called they'll try and find the port   *
* TALK otherwise they'll call Talk directly via CLI.               *
* You can call this script in your s:user-startup like this:       *
*      run rx Everyday:rexx/Talk_ARexxPort.rexx                    *
* To take advantage of this script in your events, you can set the *
* Event Execution Type to 'Talk'. This is highly recommended!!     *
*******************************************************************/

portname = upper(arg(1))

if (portname = '') then
	portname = 'TALK'

if (show('p', portname)) then
do
	say '"'portname'" port already opened!'
	exit(5)
end

if (~show('l', 'rexxsupport.library')) then
do
	if (~addlib('rexxsupport.library', 0, -30)) then
	do
		say 'failed'
		exit(10)
	end
end

if (~openport(portname)) then
do
	say 'Unable to open "'portname'" port!'
	exit(10)
end

running = 1

do while running
	waitpkt(portname)
	packet = getpkt(portname)
	
	if (packet ~= null()) then
	do
		command = getarg(packet)
		
		reply(packet, 0)
		
		if upper(command) = 'QUIT' then
			running = 0
		else
		do
			address command

			'c:talk 'command

			address rexx
		end
	end
end

if show('l', portname) then
	remlib(portname)

closeport(portname)

exit 0

/* End */
