
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <dos/dos.h>
#include <clib/dos_protos.h>
#include <clib/exec_protos.h>

#include <clib/AMarquee_protos.h>

#include <pragmas/AMarquee_pragmas.h>

struct Library * AMarqueeBase = NULL;
struct QSession * session     = NULL;

void CleanExit(void)
{
  if (session)      QFreeSession(session);       /* This MUST be done before we close the library! */
  if (AMarqueeBase) CloseLibrary(AMarqueeBase);
  printf("All done.\n");
}

/* Main program--simply increments our counter whenever someone else increments theirs */
int main(int argc, char ** argv)
{
  char * connectTo, * progName;
  int port = 2957;
  ULONG val = 0;
  BOOL BDie = FALSE;
  
  atexit(CleanExit);
    
  if (argc < 2) 
  {
    printf("Usage:  BounceMessage hostname [myname]\n");
    exit(RETURN_WARN);
  }
  connectTo = argv[1] ? argv[1] : "localhost";
  progName  = argv[2] ? argv[2] : "bouncemessage";

  if ((AMarqueeBase = OpenLibrary("amarquee.library",37L)) == NULL)
  {
    printf("Couldn't open amarquee.library v37!\n");
    exit(RETURN_ERROR);
  }
  printf("Connecting to %s:%i...\n",connectTo, port);
  if ((session = QNewSession(connectTo, port, progName)) == NULL)
  {
    printf("Couldn't connect to server %s:%i\n",connectTo, port);
    CloseLibrary(AMarqueeBase);
    exit(RETURN_WARN);
  }
  
  printf("BounceMessage connected to server %s:%i\n",connectTo, port);

  /* Setup */
  (void)QSetMessageAccessOp(session, "/#?/bounce#?", sizeof(ULONG));
  (void)QMessageOp(session, "/#?/bounce#?", &val, sizeof(ULONG));
  (void)QGo(session,0L);

  while(BDie == FALSE)
  {
    struct QMessage * qMsg;
    ULONG signals = (1L << session->qMsgPort->mp_SigBit) | (SIGBREAKF_CTRL_C);

    /* Wait for next message from the server */
    signals = Wait(signals);
    
    if (signals & (1L << session->qMsgPort->mp_SigBit))
    {
      while(qMsg = (struct QMessage *) GetMsg(session->qMsgPort))
      {
        if ((qMsg->qm_Status != QERROR_NO_ERROR)&&(qMsg->qm_Status != QERROR_UNPRIVILEGED)) 
        {
          printf("Error %i detected!\n", qMsg->qm_Status);
          BDie = TRUE;
        }
        else
        {
          /* Is it a messageOp? */
          if ((qMsg->qm_ID == 0)&&(qMsg->qm_Path)&&(qMsg->qm_Data)&&(qMsg->qm_DataLen == sizeof(ULONG)))
          {
            val = *((ULONG *)qMsg->qm_Data); 
            
            /* Raise the stakes a bit :) */
            printf("I got %lu from [%s]... ",val,qMsg->qm_Path); 
            printf("and I send back %lu!\n",++val); 

            (void)QMessageOp(session, "/#?/bounce#?", &val, sizeof(ULONG));
            (void)QGo(session,0L);
          }
          else printf("Got some weird message!\n");
        }
        FreeQMessage(session,qMsg);
      }
    }
    if (signals & SIGBREAKF_CTRL_C) BDie = TRUE;  /* Quit if CTRL-C pressed */
  }
}
